//
//	MiscStringUNIX.m
//		Written by Don Yacktman (c) 1993 by Don Yacktman.
//				Version 1.5  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/MiscString.h>

@implementation MiscString(UNIX)

// This category includes methods to support various UNIX features,
// such as file/path name parsing and encryption of passwords.

- encrypt:salt
{	// encrypt as a UNIX password using the MiscString "salt" as the salt...
	// see crypt(3) for more info
	// The cast prevents a warning:  -stringValue returns a const char *.
	// Assuming crypt() doesn't change the salt, the cast is OK.
	char *strv;
	if ([salt respondsTo:@selector(stringValue)])
		strv = (char *)[salt stringValue];
	else return nil;
	if (!(buffer && strv)) return nil;
	return [[[self class] alloc] initString:crypt(buffer, strv)];
}

- fileNameFromZone:(NXZone *)zone
{
	return [self extractPart:MISC_STRING_LAST useAsDelimiter:'/'
				 caseSensitive:YES fromZone:[self zone]];
}

- fileName
{
	return [self fileNameFromZone:[self zone]];
}

- pathNameFromZone:(NXZone *)zone
{
	return [self left:[self rspotOf:'/'] fromZone:zone];
}

- pathName
{
	return [self pathNameFromZone:[self zone]];
}

- replaceHomeWithTilde
{
	const char *home = NXHomeDirectory();
	if (![self cmp:home n:strlen(home)])
		[self replaceFrom:0 length:strlen(home) withChar:'~'];
	return self;
}

- replaceTildeWithHome
{	// ***** I'd like to see this do proper csh substitution rather than
	// just the simple implementation done here. -don
	if (buffer[0] == '~')
		[self replaceFrom:0 length:1 with:NXHomeDirectory()];
	return self;
}

@end
