
#import "ImageWell.h"

// useful macros to define what we accpet from dragging
#define dragdef const char *dragTypes[3] =\
		{ NXFilenamePboardType, NXColorPboardType, NULL }
#define dragNum	2

@implementation ImageWell

- initFrame:(const NXRect *)frm    // designated initializer for a view
{
	dragdef;
	[super initFrame:frm];
	// start off with sane values
	backColor = NX_COLORLTGRAY;
	backIsColor = YES;
	_myImage = nil;
	offset.x = 0; offset.y = 0;
	[self registerForDraggedTypes:dragTypes count:dragNum];
	dragOperation = NX_DragOperationNone;
	delegate = nil;
	return self;
}

- setDelegate:anObject { delegate = anObject; return self; }
- delegate { return delegate; }

- drawSelf:(const NXRect *)rects :(int)rectCount	// redraws the image
{
	NXSetColor(backColor);
	NXRectFill(rects);
	if (!backIsColor)
		[_myImage composite:NX_SOVER fromRect:&bounds toPoint:&offset];
	return self;
}

- image { return _myImage; }
- (const char *)imageName { return imageName; }
- (const char *)stringValue { return imageName; }

- setImageFile:(const char *)fileName
{
	NXSize tempSize, loadSize;
	double xscale, yscale, scale;
	id oldImage = _myImage;
	id tempImage = [NXImage alloc];
	if (![tempImage loadFromFile:fileName]) return nil;
	[tempImage getSize:&tempSize];
	[tempImage free];
	xscale = bounds.size.width / tempSize.width;
	yscale = bounds.size.height / tempSize.height;
	scale = ((xscale < yscale) ? xscale : yscale);
	loadSize.width = tempSize.width * scale;
	loadSize.height = tempSize.height * scale;
	offset.x = (bounds.size.width - loadSize.width) / 2;
	offset.y = (bounds.size.height - loadSize.height) / 2;
	_myImage = [[NXImage allocFromZone:[self zone]] initSize:&loadSize];
	[_myImage setBackgroundColor:backColor];
	[_myImage setScalable:YES];	// want to scale to thumbnails
	backIsColor = NO;
	if (fileName) {
		[_myImage useFromFile:fileName];
		if (imageName) free(imageName);
		imageName = (char *)malloc(strlen(fileName) + 1);
		strcpy(imageName, fileName);
	} else { // default image is to use a solid color
		_myImage = nil; backIsColor = YES;
	}
	if (_myImage) {
		[oldImage free];
	} else if (fileName) _myImage = oldImage;
	else [oldImage free];
	if (_myImage) backIsColor = NO;
	[self display];
	return _myImage;   
}

// The following two methods allow changing _myImage from
// menu items or buttons.

- changeImage:sender
{
	const char *const *types = [NXImage imageFileTypes];  
	if ([[OpenPanel new] runModalForTypes:types]) {
		[self setImageFile:[[OpenPanel new] filename]];
		[self display];
	}
	return self;
}

- acceptColor:(NXColor)color atPoint:(const NXPoint *)aPoint
{	// even though this method is "obsolete", I make sure that I call
	// it when appropriate from my dragging stuff, so it's basically
	// still around...as it ought to be.
	backIsColor = YES;
	backColor = color;
	[self update];
	return self;
}

// drag and drop an image into _myImage.
// uses 3.0 drag kit stuff

- (NXDragOperation)draggingEntered:sender
{
	NXDragOperation sourceMask;

	// Ask the sender for its operation mask.
	sourceMask = [sender draggingSourceOperationMask]; 
	if (sourceMask & NX_DragOperationCopy)
		dragOperation = NX_DragOperationCopy;
	else if (sourceMask & NX_DragOperationGeneric)
		dragOperation = NX_DragOperationGeneric;
	else dragOperation = NX_DragOperationNone;
	return dragOperation;
}	

- (NXDragOperation)draggingUpdated:sender { return dragOperation; }
- draggingExited:sender { return self; }

- (BOOL)prepareForDragOperation:sender
{ // We do want to accept the dragged image/color.
	return YES;
}
	
- (BOOL)performDragOperation:sender
{
	Pasteboard *dragPasteboard = [Pasteboard newName:NXDragPboard];
	char *file;
	int length;
	dragdef;
	BOOL foundOne = NO;

	if ([dragPasteboard findAvailableTypeFrom:dragTypes num:dragNum]
			== NXFilenamePboardType) {
		if ([dragPasteboard readType:NXFilenamePboardType
							data:&file length:&length]) {
			foundOne = ([self setImageFile:file] != nil);
			[dragPasteboard deallocatePasteboardData:file length:length];
	}	}
	else if ([dragPasteboard findAvailableTypeFrom:dragTypes num:dragNum]
			== NXColorPboardType) {
		NXPoint p = [sender draggingLocation];
		NXColor c = NXReadColorFromPasteboard([sender draggingPasteboard]);
		[self acceptColor:c atPoint:&p];
		foundOne = YES;
	}
	if (foundOne) {
		[NXApp activateSelf:YES];
		[window makeKeyAndOrderFront:self];
	}
	return foundOne;
}
	
- concludeDragOperation:sender { return self; }

@end
