/**************************************************************************
 * CLASS:		MiscImageDragView
 *
 *	See the header file for more information on this class.
 **************************************************************************/

#import <daymisckit/DAYString.h>
#import <appkit/Pasteboard.h> 
#import "MiscImageDragView.h"


@implementation MiscImageDragView

- initFrame: (NXRect *)frameRect
{	
  const char *const types[] = { NXPostScriptPboardType, NXTIFFPboardType,
  								NXFilenamePboardType };

	[super initFrame: frameRect];
	[self registerForDraggedTypes: (const char *const *)&types count: 3];
	dragIconRepresentation = YES;

	return self;
}


- awake
{
  const char *const types[] = { NXPostScriptPboardType, NXTIFFPboardType, 	
  								NXFilenamePboardType };

	[super awake];
	[self registerForDraggedTypes: (const char *const *)&types count: 3];
	
	return self;
}


- setDragIconRepresentation: (BOOL)aBool
{
	dragIconRepresentation = aBool;
	return self;
}


- (BOOL)dragIconRepresentation
{
	return dragIconRepresentation;
}


// Put the image on the correct pasteboard depending upon the type of data
// and select the image to be dragged.

- (BOOL)setupForSourceDrag
{
  BOOL wroteToPB=NO;
  id  dpb = [Pasteboard newName: NXDragPboard];
  
//	if ([self hasRepresentation: [NXEPSImageRep class] ])
//	{
		[dpb declareTypes: &NXPostScriptPboardType num: 1 owner: self];
		[self setDrawBackground: NO];
		[self writePSCodeInside: &bounds to: dragPB];
		[self setDrawBackground: YES];
		dragImage = ivImage;	
		wroteToPB = YES;
//	 }
/*	
//	if ([self hasRepresentation: [NXBitmapImageRep class] ])
//	{
//		[dragPB declareTypes: &NXTIFFPboardType num: 1 owner: self];
//printf ("tiff image\n");
//		//somehow write the image to pasteboard
//		ivImage = dragImage;	
//		wroteToPB = YES;
//	 }
	
	if (filename)
	{
		[dragPB declareTypes: &NXFilenamePboardType num: 1 owner: self];
		 [dragPB writeType: NXFilenamePboard data: [filename stringValue]
		 			length: [filename length] ];
		ivImage = dragImage;
printf ("filename image..\n");
		wroteToPB = YES;
	 }
*/	 
	return wroteToPB;
}


// Used internally to see what image represenations the current NXImage
// holds so we will know how to write it to the pasteboard if a source
// dragging session begins

- (BOOL)hasRepresentation: aClass
{
  int  i;
  id  repList;	

	if ([self image] == nil)
		return NO;
		
	repList = [ [self image] representationList];
printf ("Checking the rep list..\n");	
	for (i=0; i<[repList count]; i++)
		if ([ [repList objectAt: i] isKindOf: aClass])
			return YES;
			
	return NO;
}

		
- (BOOL)prepareForDragOperation: sender
{
	if ([NXImage canInitFromPasteboard: [sender draggingPasteboard] ])
	    return YES;

	return NO;
}


- (BOOL)performDragOperation: sender
{
	[self setImage: [ [NXImage alloc] initFromPasteboard: 
			[sender draggingPasteboard] ] ];

	return YES;
}


- read: (NXTypedStream *)stream
{
	[super read: stream];
	NXReadTypes ("c", &dragIconRepresentation);

	return self;
}


- write: (NXTypedStream *)stream
{
	[super write: stream];
	NXWriteTypes ("cc", &dragIconRepresentation);
	return self;
}

	